@@ -20,6 +20,7 @@
 """Contains and manages mappings."""
 
 from __future__ import annotations
+from evdev import ecodes
 
 import json
 import os
@@ -35,7 +36,10 @@ from typing import (
     overload,
 )
 
-from pydantic import ValidationError
+try:
+    from pydantic.v1 import ValidationError
+except ImportError:
+    from pydantic import ValidationError
 
 from inputremapper.configs.input_config import InputCombination, InputConfig
 from inputremapper.configs.mapping import Mapping, UIMapping
@@ -232,21 +236,12 @@ class Preset(Generic[MappingModel]):
             existing = self._mappings.get(permutation)
             if existing is not None:
                 return existing
-
-        logger.error(
-            "Combination %s not found. Available: %s",
-            repr(combination),
-            list(
-                self._mappings.keys(),
-            ),
-        )
-
         return None
 
     def dangerously_mapped_btn_left(self) -> bool:
         """Return True if this mapping disables BTN_Left."""
-        if InputCombination([InputConfig.btn_left()]) not in [
-            m.input_combination for m in self
+        if (ecodes.EV_KEY, ecodes.BTN_LEFT) not in [
+            m.input_combination[0].type_and_code for m in self
         ]:
             return False
 
