@@ -1,7 +1,8 @@
 """Tests that require a linux desktop environment to be running."""
-
+import tests.test
 import gi
 
+gi.require_version("Gdk", "3.0")
 gi.require_version("Gtk", "3.0")
 gi.require_version("GLib", "2.0")
 gi.require_version("GtkSource", "4")
