@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2022 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
 #
 # This file is part of input-remapper.
 #
@@ -19,13 +19,20 @@
 # along with input-remapper.  If not, see <https://www.gnu.org/licenses/>.
 
 
-from tests.test import quick_cleanup, tmp
+from tests.lib.cleanup import quick_cleanup
+from tests.lib.tmp import tmp
 
 import os
 import unittest
 import tempfile
 
-from inputremapper.configs.paths import touch, mkdir, get_preset_path, get_config_path
+from inputremapper.configs.paths import (
+    touch,
+    mkdir,
+    get_preset_path,
+    get_config_path,
+    split_all,
+)
 
 
 def _raise(error):
@@ -43,7 +50,8 @@ class TestPaths(unittest.TestCase):
             self.assertTrue(os.path.exists(path_abcde))
             self.assertTrue(os.path.isfile(path_abcde))
             self.assertRaises(
-                ValueError, lambda: touch(os.path.join(local_tmp, "a/b/c/d/f/"))
+                ValueError,
+                lambda: touch(os.path.join(local_tmp, "a/b/c/d/f/")),
             )
 
     def test_mkdir(self):
@@ -54,12 +62,15 @@ class TestPaths(unittest.TestCase):
             self.assertTrue(os.path.isdir(path_bcde))
 
     def test_get_preset_path(self):
-        self.assertEqual(get_preset_path(), os.path.join(tmp, "presets"))
-        self.assertEqual(get_preset_path("a"), os.path.join(tmp, "presets/a"))
-        self.assertEqual(
-            get_preset_path("a", "b"), os.path.join(tmp, "presets/a/b.json")
-        )
+        self.assertTrue(get_preset_path().startswith(get_config_path()))
+        self.assertTrue(get_preset_path().endswith("presets"))
+        self.assertTrue(get_preset_path("a").endswith("presets/a"))
+        self.assertTrue(get_preset_path("a", "b").endswith("presets/a/b.json"))
 
     def test_get_config_path(self):
-        self.assertEqual(get_config_path(), tmp)
-        self.assertEqual(get_config_path("a", "b"), os.path.join(tmp, "a/b"))
+        # might end with /beta_XXX
+        self.assertTrue(get_config_path().startswith(f"{tmp}/.config/input-remapper"))
+        self.assertTrue(get_config_path("a", "b").endswith("a/b"))
+
+    def test_split_all(self):
+        self.assertListEqual(split_all("a/b/c/d"), ["a", "b", "c", "d"])
