@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2022 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
 #
 # This file is part of input-remapper.
 #
@@ -17,18 +17,14 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with input-remapper.  If not, see <https://www.gnu.org/licenses/>.
+import asyncio
 
-
-from tests.test import (
-    InputDevice,
-    quick_cleanup,
-    cleanup,
-    fixtures,
-    new_event,
-    push_events,
-    EVENT_READ_TIMEOUT,
-    START_READING_DELAY,
-)
+from tests.lib.cleanup import cleanup, quick_cleanup
+from tests.lib.constants import EVENT_READ_TIMEOUT, START_READING_DELAY
+from tests.lib.logger import logger
+from tests.lib.fixtures import fixtures
+from tests.lib.pipes import push_events
+from tests.lib.patches import InputDevice
 
 import os
 import unittest
@@ -38,9 +34,12 @@ import multiprocessing
 import evdev
 from evdev.ecodes import EV_ABS, EV_KEY
 
-from inputremapper.groups import groups
-from inputremapper.gui.reader import reader
-from inputremapper.gui.helper import RootHelper
+from inputremapper.groups import groups, _Groups
+from inputremapper.gui.reader_client import ReaderClient
+from inputremapper.gui.reader_service import ReaderService
+from inputremapper.input_event import InputEvent
+from inputremapper.utils import get_device_hash
+from inputremapper.gui.messages.message_broker import MessageBroker
 
 
 class TestTest(unittest.TestCase):
@@ -69,10 +68,9 @@ class TestTest(unittest.TestCase):
         self.assertIsInstance(capabilities[EV_KEY][0], int)
 
     def test_restore_fixtures(self):
-        fixtures[1] = [1234]
-        del fixtures["/dev/input/event11"]
+        fixtures["/bar/dev"] = {"name": "bla"}
         cleanup()
-        self.assertIsNone(fixtures.get(1))
+        self.assertIsNone(fixtures.get("/bar/dev"))
         self.assertIsNotNone(fixtures.get("/dev/input/event11"))
 
     def test_restore_os_environ(self):
@@ -84,47 +82,63 @@ class TestTest(unittest.TestCase):
         self.assertNotIn("foo", environ)
 
     def test_push_events(self):
-        """Test that push_event works properly between helper and reader.
+        """Test that push_event works properly between reader service and client.
 
-        Using push_events after the helper is already forked should work,
+        Using push_events after the reader-service is already started should work,
         as well as using push_event twice
         """
+        reader_client = ReaderClient(MessageBroker(), groups)
 
-        def create_helper():
-            # this will cause pending events to be copied over to the helper
+        def create_reader_service():
+            # this will cause pending events to be copied over to the reader-service
             # process
-            def start_helper():
-                helper = RootHelper()
-                helper.run()
+            def start_reader_service():
+                # there is no point in using the global groups object
+                # because the reader-service runs in a different process
+                reader_service = ReaderService(_Groups())
+                loop = asyncio.new_event_loop()
+                loop.run_until_complete(reader_service.run())
 
-            self.helper = multiprocessing.Process(target=start_helper)
-            self.helper.start()
+            self.reader_service = multiprocessing.Process(target=start_reader_service)
+            self.reader_service.start()
             time.sleep(0.1)
 
         def wait_for_results():
-            # wait for the helper to send stuff
+            # wait for the reader-service to send stuff
             for _ in range(10):
                 time.sleep(EVENT_READ_TIMEOUT)
-                if reader._results.poll():
+                if reader_client._results_pipe.poll():
                     break
 
-        event = new_event(EV_KEY, 102, 1)
-        create_helper()
-        reader.start_reading(groups.find(key="Foo Device 2"))
+        create_reader_service()
+        reader_client.set_group(groups.find(key="Foo Device 2"))
+        reader_client.start_recorder()
         time.sleep(START_READING_DELAY)
 
-        push_events("Foo Device 2", [event])
+        event = InputEvent.key(102, 1)
+        push_events(fixtures.foo_device_2_keyboard, [event])
         wait_for_results()
-        self.assertTrue(reader._results.poll())
+        self.assertTrue(reader_client._results_pipe.poll())
 
-        reader.clear()
-        self.assertFalse(reader._results.poll())
+        reader_client._read()
+        self.assertFalse(reader_client._results_pipe.poll())
 
-        # can push more events to the helper that is inside a separate
+        # can push more events to the reader-service that is inside a separate
         # process, which end up being sent to the reader
-        push_events("Foo Device 2", [event])
+        event = InputEvent.key(102, 0)
+        logger.info("push_events")
+        push_events(fixtures.foo_device_2_keyboard, [event])
         wait_for_results()
-        self.assertTrue(reader._results.poll())
+        logger.info("assert")
+        self.assertTrue(reader_client._results_pipe.poll())
+
+        reader_client.terminate()
+
+    def test_device_hash_from_fixture_is_correct(self):
+        for fixture in fixtures:
+            self.assertEqual(
+                fixture.get_device_hash(), get_device_hash(InputDevice(fixture.path))
+            )
 
 
 if __name__ == "__main__":
